@@ -58,9 +58,10 @@ from .resolver import (get_resolver_object,
 
 from .realm import (get_realms, realm_is_defined,
                     get_default_realm,
-                    get_realm)
+                    get_realm, get_realm_id)
 from .config import get_from_config, SYSCONF
 from .usercache import (user_cache, cache_username, user_init, delete_user_cache)
+from privacyidea.models import CustomUserAttribute, db
 
 log = logging.getLogger(__name__)
 
@@ -90,6 +91,7 @@ class User(object):
         self.login = login or ""
         self.used_login = self.login
         self.realm = (realm or "").lower()
+        self.realm_id = None
         if resolver == "**":
             resolver = ""
         self.resolver = resolver or ""
@@ -101,6 +103,8 @@ class User(object):
             self._get_user_from_userstore()
             # Just store the resolver type
             self.rtype = get_resolver_type(self.resolver)
+            # Add realm_id to User object
+            self.realm_id = get_realm_id(self.realm)
 
     @user_cache(user_init)
     def _get_user_from_userstore(self):
@@ -277,8 +281,49 @@ class User(object):
         (uid, _rtype, _resolver) = self.get_user_identifiers()
         y = get_resolver_object(self.resolver)
         userInfo = y.getUserInfo(uid)
+        # Now add the custom attributes, this is used e.g. in ADDUSERINRESPONSE
+        userInfo.update(self.attributes)
         return userInfo
-    
+
+    @log_with(log)
+    def set_attribute(self, attrkey, attrvalue, attrtype=None):
+        """
+        Set a custom attribute for a user
+
+        :param attrkey: The key of the attribute
+        :param attrvalue: The value of the attribute
+        :return: The id of the attribute setting
+        """
+        ua = CustomUserAttribute(user_id=self.uid, resolver=self.resolver, realm_id=self.realm_id,
+                                 Key=attrkey, Value=attrvalue, Type=attrtype).save()
+        return ua
+
+    @property
+    def attributes(self):
+        """
+        returns the custom attributes of a user
+        :return: a dictionary of attributes with keys and values
+        """
+        return get_attributes(self.uid, self.resolver, self.realm_id)
+
+    @log_with(log)
+    def delete_attribute(self, attrkey=None):
+        """
+        Delete the given key as custom user attribute.
+        If no key is given, then all attributes are deleted
+
+        :param attrkey: The key to delete
+        :return: The number of deleted rows
+        """
+        if attrkey:
+            ua = CustomUserAttribute.query.filter_by(user_id=self.uid, resolver=self.resolver,
+                                                     realm_id=self.realm_id, Key=attrkey).delete()
+        else:
+            ua = CustomUserAttribute.query.filter_by(user_id=self.uid, resolver=self.resolver,
+                                                     realm_id=self.realm_id).delete()
+        db.session.commit()
+        return ua
+
     @log_with(log)
     def get_user_phone(self, phone_type='phone', index=None):
         """
@@ -589,7 +634,19 @@ def get_user_from_param(param, optionalOrRequired=optional):
 
 
 @log_with(log)
-def get_user_list(param=None, user=None):
+def get_user_list(param=None, user=None, custom_attributes=False):
+    """
+    This function returns a list of user dictionaries.
+
+    :param param: search parameters
+    :type param: dict
+    :param user:  a specific user object to return
+    :type user: User object
+    :param custom_attributes:  Set to True, if you want to receive custom attributes
+        of external users.
+    :type custom_attributes: bool
+    :return: list of dictionaries
+    """
     users = []
     resolvers = []
     searchDict = {"username": "*"}
@@ -648,9 +705,15 @@ def get_user_list(param=None, user=None):
             log.debug("with this search dictionary: {0!r} ".format(searchDict))
             ulist = y.getUserList(searchDict)
             # Add resolvername to the list
+            realm_id = get_realm_id(param_realm or user_realm)
             for ue in ulist:
                 ue["resolver"] = resolver_name
                 ue["editable"] = y.editable
+            if custom_attributes and realm_id is not None:
+                for ue in ulist:
+                    # Add the custom attributes, by class method from User
+                    # with uid, resolvername and realm_id, which we need to determine by the realm name
+                    ue.update(get_attributes(ue.get("userid"), ue.get("resolver"), realm_id))
             log.debug("Found this userlist: {0!r}".format(ulist))
             users.extend(ulist)
 
@@ -697,4 +760,28 @@ def log_used_user(user, other_text=""):
     :param other_text: Some additional text
     :return: str
     """
-    return u"logged in as {0}. {1}".format(user.used_login, other_text) if user.used_login != user.login else other_text
\ No newline at end of file
+    return u"logged in as {0}. {1}".format(user.used_login, other_text) if user.used_login != user.login else other_text
+
+
+def get_attributes(uid, resolver, realm_id):
+    """
+    Returns the attributes for the given user.
+
+    :param uid: The UID of the user
+    :param resolver: The name of the resolver
+    :param realm_id: The realm_id
+    :return: A dictionary of key/values
+    """
+    r = {}
+    attributes = CustomUserAttribute.query.filter_by(user_id=uid, resolver=resolver, realm_id=realm_id).all()
+    for attr in attributes:
+        r[attr.Key] = attr.Value
+    return r
+
+
+def is_attribute_at_all():
+    """
+    Check if there are custom user attributes at all
+    :return: bool
+    """
+    return bool(CustomUserAttribute.query.count())
