@@ -79,6 +79,7 @@ myApp.controller("eventDetailController", function($scope, $stateParams,
     $scope.conds = {};
     $scope.actionCheckBox = {};
     $scope.conditionCheckBox = {};
+    $scope.condition_filter = "";
     $('html,body').scrollTop(0);
 
     $scope.getEvent = function () {
@@ -225,6 +226,8 @@ myApp.controller("eventDetailController", function($scope, $stateParams,
         ConfigFactory.getHandlerConditions($scope.form.handlermodule,
             function (conditions) {
                 $scope.handlerConditions = conditions.result.value;
+                $scope.conditionGroups = []
+
                 // tick selected handlerConditions, if type===multi
                 angular.forEach($scope.handlerConditions, function(condition, name){
                      if (condition.type === "multi" && Object.keys($scope.conds).indexOf(name) >= 0) {
@@ -242,10 +245,15 @@ myApp.controller("eventDetailController", function($scope, $stateParams,
                             }
                         }
                     }
+                    if ($scope.conditionGroups.indexOf(condition.group) < 0) {
+                        $scope.conditionGroups.push(condition.group)
+                    }
                 });
             });
     };
 
+
+
     $scope.handlerModuleChanged = function () {
         $scope.getHandlerActions();
         $scope.getHandlerConditions();
