@@ -176,6 +176,10 @@ myApp.controller("tokenEnrollController", ["$scope", "TokenFactory", "$timeout",
         hashlib: "sha1",
         'radius.system_settings': true
     };
+    if ($state.includes('token.rollover')) {
+        $scope.form.serial = $stateParams.tokenSerial;
+        $scope.form.type = $stateParams.tokenType;
+    }
     $scope.vasco = {
         // Note: A primitive does not work in the ng-model of the checkbox!
         useIt: false
@@ -283,6 +287,10 @@ myApp.controller("tokenEnrollController", ["$scope", "TokenFactory", "$timeout",
             // because the user might not be allowed to list RADIUS servers
             $scope.getRADIUSIdentifiers();
         }
+        if ($scope.form.type === "remote") {
+            // fetch the privacyIDEA servers
+            $scope.getPrivacyIDEAServers();
+        }
         if ($scope.form.type === "certificate") {
             $scope.getCAConnectors();
         }
@@ -467,7 +475,9 @@ myApp.controller("tokenEnrollController", ["$scope", "TokenFactory", "$timeout",
         $scope.form.validity_period_start = date_object_to_string($scope.form.validity_period_start);
         $scope.form.validity_period_end = date_object_to_string($scope.form.validity_period_end);
         TokenFactory.enroll($scope.newUser,
-            $scope.form, $scope.callback);
+            $scope.form, $scope.callback,
+            function (data) {$scope.enrolling=false;}
+            );
     };
 
     $scope.pollTokenInfo = function () {
@@ -536,6 +546,13 @@ myApp.controller("tokenEnrollController", ["$scope", "TokenFactory", "$timeout",
         });
     };
 
+    // get the list of configured privacyIDEA server identifiers
+    $scope.getPrivacyIDEAServers = function() {
+        ConfigFactory.getPrivacyidea(function(data){
+            $scope.privacyIDEAServers = data.result.value;
+        });
+    };
+
     // get the list of configured CA connectors
     $scope.getCAConnectors = function () {
         ConfigFactory.getCAConnectorNames(function (data){
