@@ -85,23 +85,28 @@ def get_linotp_uri(config_file):
     return sql_uri
 
 
-parser = argparse.ArgumentParser(description=__doc__)
-parser.add_argument("-c", "--config",
-                    help="LinOTP config file. We only need the SQLALCHEMY_DATABASE_URI.",
-                    required=True)
-args = parser.parse_args()
+def main():
+    parser = argparse.ArgumentParser(description=__doc__)
+    parser.add_argument("-c", "--config",
+                        help="LinOTP config file. We only need the SQLALCHEMY_DATABASE_URI.",
+                        required=True)
+    args = parser.parse_args()
 
-# Parse data
+    # Parse data
 
-SQL_URI = get_linotp_uri(args.config)
+    SQL_URI = get_linotp_uri(args.config)
 
-# Start DB stuff
+    # Start DB stuff
 
-linotp_engine = create_engine(SQL_URI)
-conn_linotp = linotp_engine.connect()
+    linotp_engine = create_engine(SQL_URI)
+    conn_linotp = linotp_engine.connect()
 
-s = select([linotp_token_table.c.LinOtpTokenSerialnumber, linotp_token_table.c.LinOtpCount])
-result = conn_linotp.execute(s)
+    s = select([linotp_token_table.c.LinOtpTokenSerialnumber, linotp_token_table.c.LinOtpCount])
+    result = conn_linotp.execute(s)
 
-for r in result:
-    print(u"{0!s}, {1!s}".format(r.LinOtpTokenSerialnumber, r.LinOtpCount))
+    for r in result:
+        print(u"{0!s}, {1!s}".format(r.LinOtpTokenSerialnumber, r.LinOtpCount))
+
+
+if __name__ == '__main__':
+    main()
